#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2021/9/24 14:10
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : aiSettings.py
# @Descr   : 人体识别AI程序
# @Software: PyCharm


import os
import platform
import subprocess


class AIParser:
    # 初始化
    def __init__(self, ai_program_path):
        self.ai_program_path = ai_program_path

    # 切换工作目录
    def switch_ai_dir(self):
        # 切换AI程序目录
        if platform.system() == 'Windows':
            os.chdir(self.ai_program_path)
        else:
            os.system(self.ai_program_path)

    # 识别图片的身体25个点并输出json
    def parse_image_part(self, origin_image_path, render_image_path, json_data_path):
        self.switch_ai_dir()
        get_result_image_cmd = "OpenPose --image_dir {} --write_images {} --write_images_format jpg --write_json {} --display 0".format(
            origin_image_path,
            render_image_path, json_data_path)
        print("-----开始识别图片-----")
        print(get_result_image_cmd)
        r_v = os.system(get_result_image_cmd)
        print(r_v)
        if r_v == 0:
            print("图片识别提取成功")
        else:
            print("图片识别提取失败")
        return r_v

    # 只识别图片的身体25个点
    def parse_imag_part2(self, origin_image_path, render_image_path):
        self.switch_ai_dir()
        get_result_image_cmd = "OpenPose --image_dir {} --write_images {} --write_images_format jpg --display 0".format(
            origin_image_path,
            render_image_path)
        print("-----开始识别图片-----")
        print(get_result_image_cmd)
        r_v = os.system(get_result_image_cmd)
        print(r_v)
        if r_v == 0:
            print("图片识别提取成功")
        else:
            print("图片识别提取失败")
        return r_v

    # 只输出识别身体25个点json
    def parse_image_part3(self, origin_image_path, json_data_path):
        self.switch_ai_dir()
        get_result_image_cmd = "OpenPose --image_dir {} --write_json {} --display 0 --render_pose 0".format(
            origin_image_path,
            json_data_path)
        print("-----开始识别图片-----")
        print(get_result_image_cmd)
        r_v = os.system(get_result_image_cmd)
        print(r_v)
        if r_v == 0:
            print("图片识别提取成功")
        else:
            print("图片识别提取失败")
        return r_v

    # # 识别图片的脸部并输出json
    def parse_image_face(self, origin_image_path, render_image_path, json_data_path):
        self.switch_ai_dir()
        get_result_image_cmd = "OpenPose --image_dir {} --face --write_images {} --write_images_format jpg --write_json {} –net_resolution 320x176 --display 0".format(
            origin_image_path,
            render_image_path, json_data_path)
        print("-----开始识别图片-----")
        print(get_result_image_cmd)
        # r_v = os.system(get_result_image_cmd)
        r_v = subprocess.Popen(get_result_image_cmd, shell=True)
        print(r_v)
        if r_v == 0:
            print("图片识别提取成功")
        else:
            print("图片识别提取失败")
        return r_v

    # 识别图片的手部并输出
    def parse_image_hand(self, origin_image_path, render_image_path, json_data_path):
        self.switch_ai_dir()
        get_result_image_cmd = "OpenPose --image_dir {} --hand --write_images {} --write_images_format jpg --write_json {} --display 0".format(
            origin_image_path,
            render_image_path)
        print("-----开始识别图片-----")
        print(get_result_image_cmd)
        r_v = os.system(get_result_image_cmd)
        print(r_v)
        if r_v == 0:
            print("图片识别提取成功")
        else:
            print("图片识别提取失败")
        return r_v

    # 识别视频
    def parse_video(self):
        pass

    # 识别网络摄像头
    def parse_webcam(self):
        pass

    # 设置标注框的样式（包括字体大小，长度、宽度等），根据输入图片的大小自动计算
    def get_label_box_style(self, origin_image_width):
        # 标注字体大小
        label_font_size = round(origin_image_width / 1200 * 20)
        # 标注框的X最小值
        label_polygon_left_x = round(origin_image_width / 1200 * 5)
        # 标注框的X最大值
        label_polygon_right_x = round(origin_image_width / 1200 * 425)
        # 标注框的Y最大值
        label_polygon_top_y = round(origin_image_width / 1200 * 5)
        # 标注框的Y最小值
        label_polygon_bottom_y = round(origin_image_width / 1200 * 402)
        # 左侧或右侧标注框的高度差值
        label_size_offset = round(origin_image_width / 1200 * 280)
        # 垂直绘制红线的长度
        vertical_red_line_length = round(origin_image_width / 1200 * 1000)
        # 身体绘制的圆圈的半径
        label_body_circle_radius = round(
            origin_image_width / 1200 * 6)
        # 身体绘制的圆圈的线宽
        label_body_circle_line = round(
            origin_image_width / 1200 * 3)
        # 脸部绘制的圆圈的半径
        label_face_circle_radius = round(
            origin_image_width / 350 * 4)
        # 脸部绘制的圆圈的线宽
        label_face_circle_line = round(
            origin_image_width / 350 * 2)
        style_obj = {
            "label_font_size": label_font_size,
            "label_polygon_left_x": label_polygon_left_x,
            "label_polygon_top_y": label_polygon_top_y,
            "label_polygon_right_x": label_polygon_right_x,
            "label_polygon_bottom_y": label_polygon_bottom_y,
            "label_size_offset": label_size_offset,
            "vertical_red_line_length": vertical_red_line_length,
            "label_body_circle_radius": label_body_circle_radius,
            "label_body_circle_line": label_body_circle_line,
            "label_face_circle_radius": label_face_circle_radius,
            "label_face_circle_line": label_face_circle_line
        }
        return style_obj



"""
===================================
@Author: Yanmh
@date: Create in 2020/12/6 10:06
@description:
    sklearn2 聚类分析工具类, 包含PCA主成分算法、DBSCAN密度算法、KMeansK均值算法，MeanShift密度聚类，均值漂移聚类，linkage聚类或连接聚类，mixture高斯混合聚类
    init: in_path - 输入excel文件； out_path - 输出excel文件
    SK_PCA: fileds - 分析的字段名称
    SK_DBSCAN: fields - 分析的字段名称, eps - 半径; min_samples - 最小样本量
    SK_MEANSHIFT: fields - 分析的字段名称, bandwidth - 带宽
    SK_MIXTURE: fields - 分析的字段名称, n_components - 组件数
    KS_LINKAGE： fields - 分析的字段名称, max_d - 最大的连接距离
===================================
"""
print(__doc__)
import sys
import traceback

import pandas as pd
from scipy.cluster.hierarchy import fcluster
from scipy.cluster.hierarchy import linkage
from sklearn import mixture
from sklearn.cluster import DBSCAN
from sklearn.cluster import KMeans
from sklearn.cluster import MeanShift
from sklearn.decomposition import PCA
from vgis_log.logTools import Log


# 自定义聚类算法函数
class SklearnTool:

    def __init__(_self, in_path, out_path):
        '''初始化聚类算法'''
        print("初始化线性回归聚类算法")
        try:
            _self._log_path = out_path  # 日志文件
            _self._out_path = out_path  # 输出文件地址
            _self._data = pd.read_excel(in_path, na_values=0).fillna(0)
        except Exception:
            log_error = "读取{}文件出错了: {}".format(in_path, traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)

    def SK_PCA(_self, fields):
        '''执行聚类主成分分析算法'''
        print("PCA - 主成分分析")
        try:
            X = _self._data[fields.split(",")]
            # n_components表示保留的特征数默认为1。如果设置成‘mle’,那么会自动确定保留的特征数
            pca = PCA(n_components='mle').fit(X)
            _self._w_data = pd.DataFrame(pca.fit_transform(X))
            _self._ratio = pca.explained_variance_ratio_  # 方差
        except Exception:
            log_error = "SK_DBSCAN分析出错了: {}".format(traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)

    def SK_DBSCAN(_self, fields, eps, min_samples):
        '''执行聚类密度算法'''
        print("DBSCAN - 具有噪声的基于密度的聚类方法")
        try:
            X = _self._data[fields.split(",")]
            # 设置半径为 eps，最小样本量为 min_samples，建模
            db = DBSCAN(eps=float(eps), min_samples=float(min_samples)).fit(X)
            _self._w_data = _self._data
            _self._w_data['label'] = db.labels_  # 聚类簇结果

            _self._mean = _self._w_data.groupby('label').mean()  # k均值结果
        except Exception:
            log_error = "SK_DBSCAN分析出错了: {}".format(traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)

    def SK_KMEANS(_self, fields, num):
        '''执行K均值聚类'''
        print("SK_KMEANS - K均值聚类方法")
        try:
            X = _self._data[fields.split(",")]
            db = KMeans(n_clusters=(int)(num)).fit(X)
            _self._w_data = _self._data
            _self._w_data['label'] = db.labels_  # 聚类簇结果
        except Exception:
            log_error = "SK_KMEANS分析出错了: {}".format(traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)

    def SK_MEANSHIFT(_self, fields, bandwidth):
        '''执行密度聚类，均值漂移聚类'''
        print("SK_MEANSHIFT - 密度聚类，均值漂移聚类方法")
        try:
            X = _self._data[fields.split(",")]
            # 带宽
            db = MeanShift(bandwidth=float(bandwidth)).fit(X)
            _self._w_data = _self._data
            _self._w_data['label'] = db.labels_  # 聚类簇结果
        except Exception:
            log_error = "SK_KMEANS分析出错了: {}".format(traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)

    def SK_MIXTURE(_self, fields, components):
        '''执行高斯混合聚类'''
        print("SK_MIXTURE - 高斯混合聚类方法")
        try:
            X = _self._data[fields.split(",")]
            gmm = mixture.GaussianMixture(n_components=int(components)).fit(X)
            pred_gmm = gmm.predict(X)
            _self._w_data = _self._data
            _self._w_data['label'] = pred_gmm  # 聚类簇结果
        except Exception:
            log_error = "SK_KMEANS分析出错了: {}".format(traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)

    def KS_LINKAGE(_self, fields, max_d):
        '''层次聚类或连接聚类'''
        print("SK_LINKAGE - 层次聚类或连接聚类方法")
        try:
            X = _self._data[fields.split(",")]
            # 带宽
            ward = linkage(X, 'ward')  # 训练
            pred_h = fcluster(ward, max_d, criterion='distance')  # 预测属于哪个类
            _self._w_data = _self._data
            _self._w_data['label'] = pred_h  # 聚类簇结果
        except Exception:
            log_error = "SK_KMEANS分析出错了: {}".format(traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)

    def SK_TO_EXCEL(_self):
        '''将结果转换成excel'''
        print("聚类分析结果导出到{}".format(_self._out_path))
        try:
            _self._w_data.to_excel(_self._out_path, encoding="utf-8", index=False)
        except Exception:
            log_error = "SK_TO_EXCEL导出结果excel出错了: {}".format(traceback.format_exc())
            print(log_error)
            log = Log(_self._log_path, log_error)  # 生成日志
            log.make_log_file()
            sys.exit(2)



#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2023/7/4 14:10
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    :
# @Descr   : 样本图片裁切
# @Software: PyChar
import math
import os
from os import path
import cv2
import numpy as np


class SegmenationImageHelper:
    def __init__(self):
        pass

    # 裁剪大图到小图，按照crop_width, crop_height尺寸，按照尺寸进行外扩填充
    @staticmethod
    def seg_img_by_size_with_padding(input_img_file_path, crop_width, crop_height, crop_img_save_path, crop_file_prefix,
                                     file_ext):

        def __crop_image(img, crop_width, crop_height):
            # m是图像height，行数，y方向
            # n是图像width，列数，X方向
            m, n = img.shape[0], img.shape[1]

            # a1 是 裁剪的行数（边缘不够尺寸的也要裁剪算行数），Y方向
            a1 = (m // crop_height) + 1
            # b1 是 裁剪的列数（边缘不够尺寸的也要裁剪算列数），X方向
            b1 = (n // crop_width) + 1

            # a 是 Y 方向不够的像素
            a = a1 * crop_height - m
            # b 是 X 方向不够的像素
            b = b1 * crop_width - n

            # 将原影像填充,满足（crop_width,crop_height）的倍数，填充部分为黑色
            # 原来的Y 为13659 ，现在变成了13824，增加了a=165
            # 原来的X 为15632，现在变成了15772，增加了b=240
            img_padding = cv2.copyMakeBorder(img, 0, a, 0, b, cv2.BORDER_CONSTANT, value=(0, 0, 0))

            img_crops = []
            # Y方向
            for i in range(a1):
                # X 方向
                for j in range(b1):
                    img_c = img_padding[i * crop_height:(i + 1) * crop_height, j * crop_width:(j + 1) * crop_width]
                    img_crops.append(img_c)
            return img_crops

        # img_input = cv2.imread(input_img_file_path, 1)
        img_input = cv2.imdecode(np.fromfile(input_img_file_path, dtype=np.uint8), 1)

        img_input_segs = __crop_image(img_input, crop_width, crop_height)

        for j in range(len(img_input_segs)):
            output_file = path.join(crop_img_save_path, crop_file_prefix + '_' + str(j) + '.' + file_ext)
            # cv2.imwrite(output_file, img_input_segs[j])
            cv2.imencode("." + file_ext, img_input_segs[j])[1].tofile(output_file)
            print("生成裁剪文件:{}".format(output_file))

        print("裁剪{}*{}已保存".format(crop_width, crop_height))

    # 裁剪大图到小图，按照crop_width, crop_height尺寸，不足尺寸的保留大小
    @staticmethod
    def seg_img_by_size_with_remain(input_image_path: str, output_folder: str, crop_file_prefix: str, file_ext: str,
                                    crop_size: tuple,
                                    ) -> None:
        """
        按照制定尺寸裁切大图，如果小图不够尺寸，保留小图自己的尺寸

        @param input_image_path: 大图路径
        @param output_folder: 裁切小图所在目录，按照规定要求命令，比cropped_0.jpg,cropped_1.jpg,...cropped_n.jpg,其中n为裁切的序号
        @param crop_file_prefix:裁切图片名的前缀，比如cropped
        @param file_ext: 裁切图片的后缀
        @param crop_size: 裁切尺寸，格式为：(height,width)
        """
        print("进入大图裁剪")

        # 读取大图片
        # img = cv2.imread(input_image_path, 1)
        img = cv2.imdecode(np.fromfile(input_image_path, dtype=np.uint8), 1)

        # 图像height，行数，y方向
        # 图像width，列数，X方向
        height, width = img.shape[0], img.shape[1]
        crop_height = crop_size[0]
        crop_width = crop_size[1]

        if height <= crop_height and width <= crop_width:
            pass
        else:
            # 裁剪的行数（边缘不够尺寸的也要裁剪算行数），Y方向
            rows = math.ceil(float(height) / crop_height)
            # rows = (height // crop_height) + 1
            # 裁剪的列数（边缘不够尺寸的也要裁剪算列数），X方向
            cols = math.ceil(float(width) / crop_width)
            # cols = (width // crop_width) + 1

            # Y 方向填充的像素
            y_padd = rows * crop_height - height
            # X 方向填充的像素
            x_padd = cols * crop_width - width

            # 将原影像填充,满足裁切尺寸的倍数，填充部分为黑色
            # 原来的Y 为13659 ，现在变成了13824，增加了y_padd=165
            # 原来的X 为15632，现在变成了15772，增加了x_padd=240
            img_padding = cv2.copyMakeBorder(img, 0, y_padd, 0, x_padd, cv2.BORDER_CONSTANT, value=(0, 0, 0))

            img_crops = []
            # Y方向,循环行
            for i in range(rows):
                # X 方向,循环列
                for j in range(cols):
                    start_y = i * crop_height
                    end_y = (i + 1) * crop_height
                    start_x = j * crop_width
                    end_x = (j + 1) * crop_width
                    # 最下面的 需要去掉Y方向扩充
                    if i == rows - 1 and y_padd != 0:
                        end_y = i * crop_height + (crop_height - y_padd)
                    # 最右边的，需要去掉X方向扩展
                    if j == cols - 1 and x_padd != 0:
                        end_x = j * crop_width + (crop_width - x_padd)
                    img_c = img_padding[start_y:end_y, start_x:end_x]
                    img_crops.append(img_c)
                    # cv2.imwrite(os.path.join(output_folder, 'cropped_{}_{}.jpg'.format(i, j)), img_c)
            for t in range(len(img_crops)):
                output_file = output_folder + "/{}_{}.{}".format(crop_file_prefix, t, file_ext)
                # output_file = os.path.join(output_folder, '{}_{}.{}'.format(crop_file_prefix, t, file_ext))
                # cv2.imwrite(output_file, img_crops[t])
                cv2.imencode("." + file_ext, img_crops[t])[1].tofile(output_file)
                print("生成裁剪文件:{}".format(output_file))
            print("裁剪{}*{}已保存".format(crop_width, crop_height))

    # 将裁切后小图拼接成大图
    @staticmethod
    def concatenate_seg_images_by_size_with_remain(input_folder: str, output_image_path: str, big_image_shape: tuple,
                                                   crop_file_prefix: str, file_ext: str,
                                                   crop_size: tuple, ) -> None:
        """
        将裁切后的小图拼成大图

        @param input_folder: 裁切小图所在目录，按照规定要求命令，如cropped_0.jpg,cropped_1.jpg,...cropped_n.jpg,其中n为裁切的序号
        @param output_image_path: 拼接后的图片路径
        @param big_image_shape: 原大图尺寸，格式为： (height,width,bands)
        @param crop_file_prefix: 裁切图片名的前缀,如 cropped
        @param file_ext: 裁切图片的后缀
        @param crop_size: 裁切尺寸，格式为：(height,width)
        """

        print("进入裁剪小图拼接大图")
        # 计算裁剪后的行数和列数
        crop_height = crop_size[0]
        crop_width = crop_size[1]
        height, width = big_image_shape
        # 裁剪的行数（边缘不够尺寸的也要裁剪算行数），Y方向
        rows = (height // crop_height) + 1
        # 裁剪的列数（边缘不够尺寸的也要裁剪算列数），X方向
        cols = (width // crop_width) + 1

        # Y 方向填充的像素
        y_padd = rows * crop_height - height
        # X 方向填充的像素
        x_padd = cols * crop_width - width

        # 创建大图的空白画布
        output_img = np.zeros((height, width, 3), dtype=np.uint8)

        # Y方向
        for i in range(rows):
            # X方向
            for j in range(cols):
                #  cols是裁剪的列数，cols * i + j 表示第几个裁剪的图片提取的变化二值图（将所有裁剪图片拉平）
                cropped_image_path = os.path.join(input_folder,
                                                  '{}_'.format(crop_file_prefix) + str(
                                                      cols * i + j) + '.' + file_ext)
                # cropped_image = cv2.imread(cropped_image_path, 1)
                cropped_image = cv2.imdecode(np.fromfile(cropped_image_path, dtype=np.uint8), 1)

                start_y = i * crop_height
                end_y = (i + 1) * crop_height
                start_x = j * crop_width
                end_x = (j + 1) * crop_width
                # 最下面的 需要去掉Y方向扩充
                if i == rows - 1 and y_padd != 0:
                    end_y = i * crop_height + (crop_height - y_padd)
                # 最右边的，需要去掉X方向扩展
                if j == cols - 1 and x_padd != 0:
                    end_x = j * crop_width + (crop_width - x_padd)
                output_img[start_y:end_y, start_x:end_x] = cropped_image

        # 保存拼接后的大图
        # cv2.imwrite(output_image_path, output_img)
        cv2.imencode("." + file_ext, output_img)[1].tofile(output_image_path)
        print("拼接大图完成")


if __name__ == "__main__":
    input_img_file_path = "E:/系统开发/AI/luojianet-master/model_zoo/rs_change_detection/Building_CD_v2/CD_data/Building change detection dataset_add/1. The two-period image data/change_label/train/change_label.tif"
    crop_width = 512
    crop_height = 512
    crop_img_save_path = "E:/系统开发/AI/luojianet-master/model_zoo/rs_change_detection/Building_CD_v2/CD_data/Building change detection dataset_add/1. The two-period image data/change_label/train/splitt_images"
    crop_file_prefix = "0"
    file_ext = "tif"
    # 方法1裁切
    # SegmenationImageHelper.seg_img_by_size_with_padding(input_img_file_path, crop_width, crop_height,
    #                                                     crop_img_save_path, file_ext)

    crop_size = (crop_height, crop_width)

    # 方法2裁切
    crop_img_save_path = "E:/系统开发/AI/luojianet-master/model_zoo/rs_change_detection/Building_CD_v2/CD_data/Building change detection dataset_add/1. The two-period image data/change_label/train/spllitt_images2"
    # SegmenationImageHelper.seg_img_by_size_with_remain(input_img_file_path, crop_img_save_path, crop_file_prefix,file_ext,crop_size)

    concatenate_seg_images_save_path = "E:/系统开发/AI/luojianet-master/model_zoo/rs_change_detection/Building_CD_v2/CD_data/Building change detection dataset_add/1. The two-period image data/change_label/train/merge_image2/result.jpg"

    big_image_shape = cv2.imdecode(np.fromfile(input_img_file_path, dtype=np.uint8), -1).shape
    SegmenationImageHelper.concatenate_seg_images_by_size_with_remain(crop_img_save_path,
                                                                      concatenate_seg_images_save_path, big_image_shape,
                                                                      crop_file_prefix, file_ext, crop_size)
